package cn.estsh.i3plus.core.apiservice.configuration;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysOrganizeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserInfoService;
import cn.estsh.i3plus.core.api.iservice.busi.ISystemResourceService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.impp.framework.boot.configuration.SystemConfig;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ImppSwitchUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2019-01-11 9:55
 * @Modify:
 **/
@Component
public class AppStartSystemInit implements CommandLineRunner {

    public static final Logger LOGGER = LoggerFactory.getLogger(CommonConstWords.SYSTEM_LOG);

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private Environment environment;

    // 限制组织数量
    private static final int organizeCount = 5;
    // 限制账号数量
    private static final int userInfoCount = 100;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    private ISystemResourceService systemResourceService;

    @Autowired
    private ISysOrganizeService organizeService;

    @Autowired
    private ISysUserInfoService userInfoService;

    @Override
    public void run(String... args) throws Exception {
        try {
            LOGGER.info("【IMPP-Core开始加载基础信息...】");
            systemInitService.putAndLoadAll();

            // 加载系统配置文件
            LOGGER.info("【IMPP-Core 加载系统配置文件到缓存中 ...】");
            systemConfig.loadSystemConfig(CommonEnumUtil.SOFT_TYPE.CORE.getCode());
            LOGGER.info("【IMPP-Core 加载系统配置文件到缓存完成】");
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 检查组织数量是否超出
     */
    public void checkOrganizeCount(){
        if(organizeService.getSysOrganizeCount() > organizeCount){
            LOGGER.info("【启动检查】组织数量已超出授权许可,请联系商务人员");
            System.exit(1);
        }
    }

    /**
     * 检查组织数量是否超出
     */
    public void checkUserInfoCount(){
        if(userInfoService.getSysUserInfoCount() > userInfoCount){
            LOGGER.info("【启动检查】用户数量已超出授权许可,请联系商务人员");
            System.exit(1);
        }
    }

    /**
     * 初始化 Es 配置
     */
    public void initImppElasticSearch(){
        try {
            if(ImppSwitchUtil.isElasticSearchSwitch()){
                String rul = environment.getProperty(CommonConstWords.PROP_IMPP_ELASTICSEARCH_URL);
                if(StringUtils.isNotBlank(rul)){
                    ElasticSearchTool.init(rul);
                }else {
                    LOGGER.info("【启动加载】启动加载 ElasticSearch 组件错误,url 参数缺失");
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            LOGGER.info("【启动加载】启动加载 ElasticSearch Error Message:{}",e.getMessage());
        }
    }

}
